--------------------------------------------------------------------------
--
-- IDECPLD 
--
-- IDE Interface for DBox2
--
-- (C) 2006 Uli Tessel (aka DboxBaer)
-- (C) 2008 kpt.ahab & Bastian Rehberg (aka Stingray) fixed for Sagem
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; (version 2 of the License)
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--
--------------------------------------------------------------------------
--
-- dataReg:   
-- the central data register the CPU interacts with
-- 
-- implemented as a 3 Port 32 Bit Mux with 32 D-FlipFlops
-- 
--------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity DataReg is
  Port (
	 A: in STD_LOGIC_VECTOR(31 downto 0);
	 B: in STD_LOGIC_VECTOR(31 downto 0);	
	 C: in STD_LOGIC_VECTOR(31 downto 0);		
    load: in STD_LOGIC;
	 mux: in STD_LOGIC_VECTOR(1 downto 0);	 
	 output: out STD_LOGIC_VECTOR(31 downto 0);
	 reset: in  STD_LOGIC;
    clk : in  STD_LOGIC );
end DataReg;

architecture Behavioral of DataReg is

  signal muxOut: STD_LOGIC_VECTOR(31 downto 0);
  signal data: STD_LOGIC_VECTOR(31 downto 0);

begin
  process (A,B,C,mux)
  begin
    case (mux) is
	  when "00" => muxOut <= A;
	  when "01" => muxOut <= B;
	  when "10" => muxOut <= C;
	  when others => muxOut <= (others=>'0');
	 end case;
  end process;

  process (clk, Reset, load, muxOut)
  begin
    if (Reset = '0') then
      data  <= (others => '0');		
    elsif (clk'event and clk = '1') then

	   if (load = '1' ) then
		  data <= muxOut;
  	   end if;
		
 	 end if;
  end process;
  
  output <= data;

end Behavioral;

